-file("/Users/cam/.asdf/installs/erlang/24.2.1/lib/parsetools-2.3.2/include/leexinc.hrl", 0).
%% The source of this file is part of leex distribution, as such it
%% has the same Copyright as the other files in the leex
%% distribution. The Copyright is defined in the accompanying file
%% COPYRIGHT. However, the resultant scanner generated by leex is the
%% property of the creator of the scanner and is not covered by that
%% Copyright.

-module(smart_move_leex).

-export([string/1,string/2,token/2,token/3,tokens/2,tokens/3]).
-export([format_error/1]).

%% User code. This is placed here to allow extra attributes.
-file("src/smart_move_leex.xrl", 127).

parse_module(TokenChars) ->
   R1 = trim(TokenChars),
   trim(R1, leading, "module").

parse_brace(TokenChars) ->
    trim(TokenChars).

otp_ver() ->
    {match, [VerStr]} = re:run(erlang:system_info(otp_release), "^R?([0-9]+)", [{capture,[1],list}]),
    list_to_integer(VerStr).

has_string_trim() ->
    otp_ver() >= 20.

trim(S) -> trim(S, both).
trim(S, D) ->
    case has_string_trim() of
        true -> string:trim(S, D);
        false -> my_trim(S, D, " ")
    end.

trim(S, D, Chars) ->
    case has_string_trim() of
        true -> string:trim(S, D, Chars);
        false -> my_trim(S, D, Chars)
    end.

my_trim([S], D, List) when is_list(S) ->
    my_trim(S, D, List);

my_trim(S, D, [Sep]) ->
    D2 = case D of
        leading -> left;
        trailing -> right;
        both -> both
    end,
    string:strip(S, D2, Sep);
my_trim(S, leading, List) ->
    L = length(List),
    case string:substr(S, 1, L) of
        List -> string:substr(S, L + 1);
        _ -> S
    end.
-file("/Users/cam/.asdf/installs/erlang/24.2.1/lib/parsetools-2.3.2/include/leexinc.hrl", 14).

format_error({illegal,S}) -> ["illegal characters ",io_lib:write_string(S)];
format_error({user,S}) -> S.

string(String) -> string(String, 1).

string(String, Line) -> string(String, Line, String, []).

%% string(InChars, Line, TokenChars, Tokens) ->
%% {ok,Tokens,Line} | {error,ErrorInfo,Line}.
%% Note the line number going into yystate, L0, is line of token
%% start while line number returned is line of token end. We want line
%% of token start.

string([], L, [], Ts) ->                     % No partial tokens!
    {ok,yyrev(Ts),L};
string(Ics0, L0, Tcs, Ts) ->
    case yystate(yystate(), Ics0, L0, 0, reject, 0) of
        {A,Alen,Ics1,L1} ->                  % Accepting end state
            string_cont(Ics1, L1, yyaction(A, Alen, Tcs, L0), Ts);
        {A,Alen,Ics1,L1,_S1} ->              % Accepting transistion state
            string_cont(Ics1, L1, yyaction(A, Alen, Tcs, L0), Ts);
        {reject,_Alen,Tlen,_Ics1,L1,_S1} ->  % After a non-accepting state
            {error,{L0,?MODULE,{illegal,yypre(Tcs, Tlen+1)}},L1};
        {A,Alen,Tlen,_Ics1,L1,_S1} ->
            Tcs1 = yysuf(Tcs, Alen),
            L2 = adjust_line(Tlen, Alen, Tcs1, L1),
            string_cont(Tcs1, L2, yyaction(A, Alen, Tcs, L0), Ts)
    end.

%% string_cont(RestChars, Line, Token, Tokens)
%% Test for and remove the end token wrapper. Push back characters
%% are prepended to RestChars.

-dialyzer({nowarn_function, string_cont/4}).

string_cont(Rest, Line, {token,T}, Ts) ->
    string(Rest, Line, Rest, [T|Ts]);
string_cont(Rest, Line, {token,T,Push}, Ts) ->
    NewRest = Push ++ Rest,
    string(NewRest, Line, NewRest, [T|Ts]);
string_cont(Rest, Line, {end_token,T}, Ts) ->
    string(Rest, Line, Rest, [T|Ts]);
string_cont(Rest, Line, {end_token,T,Push}, Ts) ->
    NewRest = Push ++ Rest,
    string(NewRest, Line, NewRest, [T|Ts]);
string_cont(Rest, Line, skip_token, Ts) ->
    string(Rest, Line, Rest, Ts);
string_cont(Rest, Line, {skip_token,Push}, Ts) ->
    NewRest = Push ++ Rest,
    string(NewRest, Line, NewRest, Ts);
string_cont(_Rest, Line, {error,S}, _Ts) ->
    {error,{Line,?MODULE,{user,S}},Line}.

%% token(Continuation, Chars) ->
%% token(Continuation, Chars, Line) ->
%% {more,Continuation} | {done,ReturnVal,RestChars}.
%% Must be careful when re-entering to append the latest characters to the
%% after characters in an accept. The continuation is:
%% {token,State,CurrLine,TokenChars,TokenLen,TokenLine,AccAction,AccLen}

token(Cont, Chars) -> token(Cont, Chars, 1).

token([], Chars, Line) ->
    token(yystate(), Chars, Line, Chars, 0, Line, reject, 0);
token({token,State,Line,Tcs,Tlen,Tline,Action,Alen}, Chars, _) ->
    token(State, Chars, Line, Tcs ++ Chars, Tlen, Tline, Action, Alen).

%% token(State, InChars, Line, TokenChars, TokenLen, TokenLine,
%% AcceptAction, AcceptLen) ->
%% {more,Continuation} | {done,ReturnVal,RestChars}.
%% The argument order is chosen to be more efficient.

token(S0, Ics0, L0, Tcs, Tlen0, Tline, A0, Alen0) ->
    case yystate(S0, Ics0, L0, Tlen0, A0, Alen0) of
        %% Accepting end state, we have a token.
        {A1,Alen1,Ics1,L1} ->
            token_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, Tline));
        %% Accepting transition state, can take more chars.
        {A1,Alen1,[],L1,S1} ->                  % Need more chars to check
            {more,{token,S1,L1,Tcs,Alen1,Tline,A1,Alen1}};
        {A1,Alen1,Ics1,L1,_S1} ->               % Take what we got
            token_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, Tline));
        %% After a non-accepting state, maybe reach accept state later.
        {A1,Alen1,Tlen1,[],L1,S1} ->            % Need more chars to check
            {more,{token,S1,L1,Tcs,Tlen1,Tline,A1,Alen1}};
        {reject,_Alen1,Tlen1,eof,L1,_S1} ->     % No token match
            %% Check for partial token which is error.
            Ret = if Tlen1 > 0 -> {error,{Tline,?MODULE,
                                          %% Skip eof tail in Tcs.
                                          {illegal,yypre(Tcs, Tlen1)}},L1};
                     true -> {eof,L1}
                  end,
            {done,Ret,eof};
        {reject,_Alen1,Tlen1,Ics1,L1,_S1} ->    % No token match
            Error = {Tline,?MODULE,{illegal,yypre(Tcs, Tlen1+1)}},
            {done,{error,Error,L1},Ics1};
        {A1,Alen1,Tlen1,_Ics1,L1,_S1} ->       % Use last accept match
            Tcs1 = yysuf(Tcs, Alen1),
            L2 = adjust_line(Tlen1, Alen1, Tcs1, L1),
            token_cont(Tcs1, L2, yyaction(A1, Alen1, Tcs, Tline))
    end.

%% token_cont(RestChars, Line, Token)
%% If we have a token or error then return done, else if we have a
%% skip_token then continue.

-dialyzer({nowarn_function, token_cont/3}).

token_cont(Rest, Line, {token,T}) ->
    {done,{ok,T,Line},Rest};
token_cont(Rest, Line, {token,T,Push}) ->
    NewRest = Push ++ Rest,
    {done,{ok,T,Line},NewRest};
token_cont(Rest, Line, {end_token,T}) ->
    {done,{ok,T,Line},Rest};
token_cont(Rest, Line, {end_token,T,Push}) ->
    NewRest = Push ++ Rest,
    {done,{ok,T,Line},NewRest};
token_cont(Rest, Line, skip_token) ->
    token(yystate(), Rest, Line, Rest, 0, Line, reject, 0);
token_cont(Rest, Line, {skip_token,Push}) ->
    NewRest = Push ++ Rest,
    token(yystate(), NewRest, Line, NewRest, 0, Line, reject, 0);
token_cont(Rest, Line, {error,S}) ->
    {done,{error,{Line,?MODULE,{user,S}},Line},Rest}.

%% tokens(Continuation, Chars, Line) ->
%% {more,Continuation} | {done,ReturnVal,RestChars}.
%% Must be careful when re-entering to append the latest characters to the
%% after characters in an accept. The continuation is:
%% {tokens,State,CurrLine,TokenChars,TokenLen,TokenLine,Tokens,AccAction,AccLen}
%% {skip_tokens,State,CurrLine,TokenChars,TokenLen,TokenLine,Error,AccAction,AccLen}

tokens(Cont, Chars) -> tokens(Cont, Chars, 1).

tokens([], Chars, Line) ->
    tokens(yystate(), Chars, Line, Chars, 0, Line, [], reject, 0);
tokens({tokens,State,Line,Tcs,Tlen,Tline,Ts,Action,Alen}, Chars, _) ->
    tokens(State, Chars, Line, Tcs ++ Chars, Tlen, Tline, Ts, Action, Alen);
tokens({skip_tokens,State,Line,Tcs,Tlen,Tline,Error,Action,Alen}, Chars, _) ->
    skip_tokens(State, Chars, Line, Tcs ++ Chars, Tlen, Tline, Error, Action, Alen).

%% tokens(State, InChars, Line, TokenChars, TokenLen, TokenLine, Tokens,
%% AcceptAction, AcceptLen) ->
%% {more,Continuation} | {done,ReturnVal,RestChars}.

tokens(S0, Ics0, L0, Tcs, Tlen0, Tline, Ts, A0, Alen0) ->
    case yystate(S0, Ics0, L0, Tlen0, A0, Alen0) of
        %% Accepting end state, we have a token.
        {A1,Alen1,Ics1,L1} ->
            tokens_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, Tline), Ts);
        %% Accepting transition state, can take more chars.
        {A1,Alen1,[],L1,S1} ->                  % Need more chars to check
            {more,{tokens,S1,L1,Tcs,Alen1,Tline,Ts,A1,Alen1}};
        {A1,Alen1,Ics1,L1,_S1} ->               % Take what we got
            tokens_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, Tline), Ts);
        %% After a non-accepting state, maybe reach accept state later.
        {A1,Alen1,Tlen1,[],L1,S1} ->            % Need more chars to check
            {more,{tokens,S1,L1,Tcs,Tlen1,Tline,Ts,A1,Alen1}};
        {reject,_Alen1,Tlen1,eof,L1,_S1} ->     % No token match
            %% Check for partial token which is error, no need to skip here.
            Ret = if Tlen1 > 0 -> {error,{Tline,?MODULE,
                                          %% Skip eof tail in Tcs.
                                          {illegal,yypre(Tcs, Tlen1)}},L1};
                     Ts == [] -> {eof,L1};
                     true -> {ok,yyrev(Ts),L1}
                  end,
            {done,Ret,eof};
        {reject,_Alen1,Tlen1,_Ics1,L1,_S1} ->
            %% Skip rest of tokens.
            Error = {L1,?MODULE,{illegal,yypre(Tcs, Tlen1+1)}},
            skip_tokens(yysuf(Tcs, Tlen1+1), L1, Error);
        {A1,Alen1,Tlen1,_Ics1,L1,_S1} ->
            Token = yyaction(A1, Alen1, Tcs, Tline),
            Tcs1 = yysuf(Tcs, Alen1),
            L2 = adjust_line(Tlen1, Alen1, Tcs1, L1),
            tokens_cont(Tcs1, L2, Token, Ts)
    end.

%% tokens_cont(RestChars, Line, Token, Tokens)
%% If we have an end_token or error then return done, else if we have
%% a token then save it and continue, else if we have a skip_token
%% just continue.

-dialyzer({nowarn_function, tokens_cont/4}).

tokens_cont(Rest, Line, {token,T}, Ts) ->
    tokens(yystate(), Rest, Line, Rest, 0, Line, [T|Ts], reject, 0);
tokens_cont(Rest, Line, {token,T,Push}, Ts) ->
    NewRest = Push ++ Rest,
    tokens(yystate(), NewRest, Line, NewRest, 0, Line, [T|Ts], reject, 0);
tokens_cont(Rest, Line, {end_token,T}, Ts) ->
    {done,{ok,yyrev(Ts, [T]),Line},Rest};
tokens_cont(Rest, Line, {end_token,T,Push}, Ts) ->
    NewRest = Push ++ Rest,
    {done,{ok,yyrev(Ts, [T]),Line},NewRest};
tokens_cont(Rest, Line, skip_token, Ts) ->
    tokens(yystate(), Rest, Line, Rest, 0, Line, Ts, reject, 0);
tokens_cont(Rest, Line, {skip_token,Push}, Ts) ->
    NewRest = Push ++ Rest,
    tokens(yystate(), NewRest, Line, NewRest, 0, Line, Ts, reject, 0);
tokens_cont(Rest, Line, {error,S}, _Ts) ->
    skip_tokens(Rest, Line, {Line,?MODULE,{user,S}}).

%%skip_tokens(InChars, Line, Error) -> {done,{error,Error,Line},Ics}.
%% Skip tokens until an end token, junk everything and return the error.

skip_tokens(Ics, Line, Error) ->
    skip_tokens(yystate(), Ics, Line, Ics, 0, Line, Error, reject, 0).

%% skip_tokens(State, InChars, Line, TokenChars, TokenLen, TokenLine, Tokens,
%% AcceptAction, AcceptLen) ->
%% {more,Continuation} | {done,ReturnVal,RestChars}.

skip_tokens(S0, Ics0, L0, Tcs, Tlen0, Tline, Error, A0, Alen0) ->
    case yystate(S0, Ics0, L0, Tlen0, A0, Alen0) of
        {A1,Alen1,Ics1,L1} ->                  % Accepting end state
            skip_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, Tline), Error);
        {A1,Alen1,[],L1,S1} ->                 % After an accepting state
            {more,{skip_tokens,S1,L1,Tcs,Alen1,Tline,Error,A1,Alen1}};
        {A1,Alen1,Ics1,L1,_S1} ->
            skip_cont(Ics1, L1, yyaction(A1, Alen1, Tcs, Tline), Error);
        {A1,Alen1,Tlen1,[],L1,S1} ->           % After a non-accepting state
            {more,{skip_tokens,S1,L1,Tcs,Tlen1,Tline,Error,A1,Alen1}};
        {reject,_Alen1,_Tlen1,eof,L1,_S1} ->
            {done,{error,Error,L1},eof};
        {reject,_Alen1,Tlen1,_Ics1,L1,_S1} ->
            skip_tokens(yysuf(Tcs, Tlen1+1), L1, Error);
        {A1,Alen1,Tlen1,_Ics1,L1,_S1} ->
            Token = yyaction(A1, Alen1, Tcs, Tline),
            Tcs1 = yysuf(Tcs, Alen1),
            L2 = adjust_line(Tlen1, Alen1, Tcs1, L1),
            skip_cont(Tcs1, L2, Token, Error)
    end.

%% skip_cont(RestChars, Line, Token, Error)
%% Skip tokens until we have an end_token or error then return done
%% with the original rror.

-dialyzer({nowarn_function, skip_cont/4}).

skip_cont(Rest, Line, {token,_T}, Error) ->
    skip_tokens(yystate(), Rest, Line, Rest, 0, Line, Error, reject, 0);
skip_cont(Rest, Line, {token,_T,Push}, Error) ->
    NewRest = Push ++ Rest,
    skip_tokens(yystate(), NewRest, Line, NewRest, 0, Line, Error, reject, 0);
skip_cont(Rest, Line, {end_token,_T}, Error) ->
    {done,{error,Error,Line},Rest};
skip_cont(Rest, Line, {end_token,_T,Push}, Error) ->
    NewRest = Push ++ Rest,
    {done,{error,Error,Line},NewRest};
skip_cont(Rest, Line, skip_token, Error) ->
    skip_tokens(yystate(), Rest, Line, Rest, 0, Line, Error, reject, 0);
skip_cont(Rest, Line, {skip_token,Push}, Error) ->
    NewRest = Push ++ Rest,
    skip_tokens(yystate(), NewRest, Line, NewRest, 0, Line, Error, reject, 0);
skip_cont(Rest, Line, {error,_S}, Error) ->
    skip_tokens(yystate(), Rest, Line, Rest, 0, Line, Error, reject, 0).

-compile({nowarn_unused_function, [yyrev/1, yyrev/2, yypre/2, yysuf/2]}).

yyrev(List) -> lists:reverse(List).
yyrev(List, Tail) -> lists:reverse(List, Tail).
yypre(List, N) -> lists:sublist(List, N).
yysuf(List, N) -> lists:nthtail(N, List).

%% adjust_line(TokenLength, AcceptLength, Chars, Line) -> NewLine
%% Make sure that newlines in Chars are not counted twice.
%% Line has been updated with respect to newlines in the prefix of
%% Chars consisting of (TokenLength - AcceptLength) characters.

-compile({nowarn_unused_function, adjust_line/4}).

adjust_line(N, N, _Cs, L) -> L;
adjust_line(T, A, [$\n|Cs], L) ->
    adjust_line(T-1, A, Cs, L-1);
adjust_line(T, A, [_|Cs], L) ->
    adjust_line(T-1, A, Cs, L).

%% yystate() -> InitialState.
%% yystate(State, InChars, Line, CurrTokLen, AcceptAction, AcceptLen) ->
%% {Action, AcceptLen, RestChars, Line} |
%% {Action, AcceptLen, RestChars, Line, State} |
%% {reject, AcceptLen, CurrTokLen, RestChars, Line, State} |
%% {Action, AcceptLen, CurrTokLen, RestChars, Line, State}.
%% Generated state transition functions. The non-accepting end state
%% return signal either an unrecognised character or end of current
%% input.

-file("src/smart_move_leex.erl", 351).
yystate() -> 54.

yystate(117, [123|Ics], Line, Tlen, _, _) ->
    yystate(53, Ics, Line, Tlen+1, 21, Tlen);
yystate(117, [47|Ics], Line, Tlen, _, _) ->
    yystate(55, Ics, Line, Tlen+1, 21, Tlen);
yystate(117, [42|Ics], Line, Tlen, _, _) ->
    yystate(117, Ics, Line, Tlen+1, 21, Tlen);
yystate(117, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(117, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(117, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(117, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(117, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(117, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,117};
yystate(116, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(116, [110|Ics], Line, Tlen, _, _) ->
    yystate(64, Ics, Line, Tlen+1, 21, Tlen);
yystate(116, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(116, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(116, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(116, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 109 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(116, [C|Ics], Line, Tlen, _, _) when C >= 111, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(116, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(116, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,116};
yystate(115, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(115, [47|Ics], Line, Tlen, _, _) ->
    yystate(51, Ics, Line, Tlen+1, 21, Tlen);
yystate(115, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(115, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(115, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(115, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(115, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(115, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(115, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,115};
yystate(114, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(114, [117|Ics], Line, Tlen, _, _) ->
    yystate(50, Ics, Line, Tlen+1, 21, Tlen);
yystate(114, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(114, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(114, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(114, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 116 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(114, [C|Ics], Line, Tlen, _, _) when C >= 118, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(114, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(114, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,114};
yystate(113, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, [116|Ics], Line, Tlen, _, _) ->
    yystate(59, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, [99|Ics], Line, Tlen, _, _) ->
    yystate(30, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 98 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, [C|Ics], Line, Tlen, _, _) when C >= 100, C =< 115 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, [C|Ics], Line, Tlen, _, _) when C >= 117, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(113, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,113};
yystate(112, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(112, [110|Ics], Line, Tlen, _, _) ->
    yystate(60, Ics, Line, Tlen+1, 21, Tlen);
yystate(112, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(112, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(112, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(112, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 109 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(112, [C|Ics], Line, Tlen, _, _) when C >= 111, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(112, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(112, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,112};
yystate(111, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(111, [110|Ics], Line, Tlen, _, _) ->
    yystate(47, Ics, Line, Tlen+1, 21, Tlen);
yystate(111, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(111, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(111, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(111, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 109 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(111, [C|Ics], Line, Tlen, _, _) when C >= 111, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(111, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(111, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,111};
yystate(110, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(110, [112|Ics], Line, Tlen, _, _) ->
    yystate(46, Ics, Line, Tlen+1, 21, Tlen);
yystate(110, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(110, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(110, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(110, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 111 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(110, [C|Ics], Line, Tlen, _, _) when C >= 113, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(110, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(110, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,110};
yystate(109, [123|Ics], Line, Tlen, _, _) ->
    yystate(45, Ics, Line, Tlen+1, 2, Tlen);
yystate(109, [42|Ics], Line, Tlen, _, _) ->
    yystate(63, Ics, Line, Tlen+1, 2, Tlen);
yystate(109, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(45, Ics, Line, Tlen+1, 2, Tlen);
yystate(109, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(45, Ics, Line, Tlen+1, 2, Tlen);
yystate(109, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(45, Ics, Line, Tlen+1, 2, Tlen);
yystate(109, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(45, Ics, Line, Tlen+1, 2, Tlen);
yystate(109, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,109};
yystate(108, [123|Ics], Line, Tlen, _, _) ->
    yystate(103, Ics, Line, Tlen+1, 16, Tlen);
yystate(108, [59|Ics], Line, Tlen, _, _) ->
    yystate(39, Ics, Line, Tlen+1, 16, Tlen);
yystate(108, [42|Ics], Line, Tlen, _, _) ->
    yystate(24, Ics, Line, Tlen+1, 16, Tlen);
yystate(108, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(108, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(108, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 58 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(108, [C|Ics], Line, Tlen, _, _) when C >= 60, C =< 122 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(108, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(108, Ics, Line, Tlen, _, _) ->
    {16,Tlen,Ics,Line,108};
yystate(107, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(107, [108|Ics], Line, Tlen, _, _) ->
    yystate(65, Ics, Line, Tlen+1, 21, Tlen);
yystate(107, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(107, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(107, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(107, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 107 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(107, [C|Ics], Line, Tlen, _, _) when C >= 109, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(107, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(107, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,107};
yystate(106, [123|Ics], Line, Tlen, _, _) ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(106, [47|Ics], Line, Tlen, _, _) ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(106, [42|Ics], Line, Tlen, _, _) ->
    yystate(66, Ics, Line, Tlen+1, 1, Tlen);
yystate(106, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(106, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(106, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(106, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(106, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(106, Ics, Line, Tlen, _, _) ->
    {1,Tlen,Ics,Line,106};
yystate(105, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(105, [101|Ics], Line, Tlen, _, _) ->
    yystate(2, Ics, Line, Tlen+1, 21, Tlen);
yystate(105, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(105, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(105, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(105, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 100 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(105, [C|Ics], Line, Tlen, _, _) when C >= 102, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(105, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(105, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,105};
yystate(104, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(104, [116|Ics], Line, Tlen, _, _) ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(104, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(104, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(104, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(104, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 115 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(104, [C|Ics], Line, Tlen, _, _) when C >= 117, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(104, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(104, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,104};
yystate(103, [123|Ics], Line, Tlen, _, _) ->
    yystate(103, Ics, Line, Tlen+1, 13, Tlen);
yystate(103, [42|Ics], Line, Tlen, _, _) ->
    yystate(24, Ics, Line, Tlen+1, 13, Tlen);
yystate(103, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(44, Ics, Line, Tlen+1, 13, Tlen);
yystate(103, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(44, Ics, Line, Tlen+1, 13, Tlen);
yystate(103, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(44, Ics, Line, Tlen+1, 13, Tlen);
yystate(103, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(44, Ics, Line, Tlen+1, 13, Tlen);
yystate(103, Ics, Line, Tlen, _, _) ->
    {13,Tlen,Ics,Line,103};
yystate(102, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(102, [101|Ics], Line, Tlen, _, _) ->
    yystate(70, Ics, Line, Tlen+1, 21, Tlen);
yystate(102, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(102, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(102, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(102, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 100 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(102, [C|Ics], Line, Tlen, _, _) when C >= 102, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(102, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(102, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,102};
yystate(101, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 19, Tlen);
yystate(101, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 19, Tlen);
yystate(101, [13|Ics], Line, Tlen, _, _) ->
    yystate(101, Ics, Line, Tlen+1, 19, Tlen);
yystate(101, [11|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 19, Tlen);
yystate(101, [12|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 19, Tlen);
yystate(101, [10|Ics], Line, Tlen, _, _) ->
    yystate(37, Ics, Line+1, Tlen+1, 19, Tlen);
yystate(101, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 19, Tlen);
yystate(101, [C|Ics], Line, Tlen, _, _) when C >= 14, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 19, Tlen);
yystate(101, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 19, Tlen);
yystate(101, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 19, Tlen);
yystate(101, Ics, Line, Tlen, _, _) ->
    {19,Tlen,Ics,Line,101};
yystate(100, [123|Ics], Line, Tlen, _, _) ->
    yystate(27, Ics, Line, Tlen+1, 2, Tlen);
yystate(100, [42|Ics], Line, Tlen, _, _) ->
    yystate(80, Ics, Line, Tlen+1, 2, Tlen);
yystate(100, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(27, Ics, Line, Tlen+1, 2, Tlen);
yystate(100, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(27, Ics, Line, Tlen+1, 2, Tlen);
yystate(100, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(27, Ics, Line, Tlen+1, 2, Tlen);
yystate(100, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(27, Ics, Line, Tlen+1, 2, Tlen);
yystate(100, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,100};
yystate(99, [125|Ics], Line, Tlen, _, _) ->
    yystate(99, Ics, Line, Tlen+1, 11, Tlen);
yystate(99, [124|Ics], Line, Tlen, _, _) ->
    yystate(73, Ics, Line, Tlen+1, 11, Tlen);
yystate(99, [123|Ics], Line, Tlen, _, _) ->
    yystate(73, Ics, Line, Tlen+1, 11, Tlen);
yystate(99, [42|Ics], Line, Tlen, _, _) ->
    yystate(35, Ics, Line, Tlen+1, 11, Tlen);
yystate(99, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(73, Ics, Line, Tlen+1, 11, Tlen);
yystate(99, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(73, Ics, Line, Tlen+1, 11, Tlen);
yystate(99, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(73, Ics, Line, Tlen+1, 11, Tlen);
yystate(99, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(73, Ics, Line, Tlen+1, 11, Tlen);
yystate(99, Ics, Line, Tlen, _, _) ->
    {11,Tlen,Ics,Line,99};
yystate(98, [123|Ics], Line, Tlen, _, _) ->
    yystate(34, Ics, Line, Tlen+1, 6, Tlen);
yystate(98, [59|Ics], Line, Tlen, _, _) ->
    yystate(98, Ics, Line, Tlen+1, 6, Tlen);
yystate(98, [42|Ics], Line, Tlen, _, _) ->
    yystate(74, Ics, Line, Tlen+1, 6, Tlen);
yystate(98, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(49, Ics, Line, Tlen+1, 6, Tlen);
yystate(98, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(49, Ics, Line, Tlen+1, 6, Tlen);
yystate(98, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 58 ->
    yystate(49, Ics, Line, Tlen+1, 6, Tlen);
yystate(98, [C|Ics], Line, Tlen, _, _) when C >= 60, C =< 122 ->
    yystate(49, Ics, Line, Tlen+1, 6, Tlen);
yystate(98, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(49, Ics, Line, Tlen+1, 6, Tlen);
yystate(98, Ics, Line, Tlen, _, _) ->
    {6,Tlen,Ics,Line,98};
yystate(97, [125|Ics], Line, Tlen, _, _) ->
    yystate(90, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [124|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [123|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [42|Ics], Line, Tlen, _, _) ->
    yystate(26, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [32|Ics], Line, Tlen, _, _) ->
    yystate(97, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [9|Ics], Line, Tlen, _, _) ->
    yystate(97, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(97, Ics, Line, Tlen, _, _) ->
    {12,Tlen,Ics,Line,97};
yystate(96, [123|Ics], Line, Tlen, _, _) ->
    yystate(76, Ics, Line, Tlen+1, 18, Tlen);
yystate(96, [47|Ics], Line, Tlen, _, _) ->
    yystate(40, Ics, Line, Tlen+1, 18, Tlen);
yystate(96, [42|Ics], Line, Tlen, _, _) ->
    yystate(96, Ics, Line, Tlen+1, 18, Tlen);
yystate(96, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(7, Ics, Line, Tlen+1, 18, Tlen);
yystate(96, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(7, Ics, Line, Tlen+1, 18, Tlen);
yystate(96, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(7, Ics, Line, Tlen+1, 18, Tlen);
yystate(96, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(7, Ics, Line, Tlen+1, 18, Tlen);
yystate(96, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(7, Ics, Line, Tlen+1, 18, Tlen);
yystate(96, Ics, Line, Tlen, _, _) ->
    {18,Tlen,Ics,Line,96};
yystate(95, [123|Ics], Line, Tlen, _, _) ->
    yystate(95, Ics, Line, Tlen+1, 5, Tlen);
yystate(95, [42|Ics], Line, Tlen, _, _) ->
    yystate(31, Ics, Line, Tlen+1, 5, Tlen);
yystate(95, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(84, Ics, Line, Tlen+1, 5, Tlen);
yystate(95, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(84, Ics, Line, Tlen+1, 5, Tlen);
yystate(95, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(84, Ics, Line, Tlen+1, 5, Tlen);
yystate(95, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(84, Ics, Line, Tlen+1, 5, Tlen);
yystate(95, Ics, Line, Tlen, _, _) ->
    {5,Tlen,Ics,Line,95};
yystate(94, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(94, [114|Ics], Line, Tlen, _, _) ->
    yystate(23, Ics, Line, Tlen+1, 21, Tlen);
yystate(94, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(94, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(94, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(94, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 113 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(94, [C|Ics], Line, Tlen, _, _) when C >= 115, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(94, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(94, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,94};
yystate(93, [125|Ics], Line, Tlen, _, _) ->
    yystate(15, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, [124|Ics], Line, Tlen, _, _) ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, [123|Ics], Line, Tlen, _, _) ->
    yystate(79, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, [47|Ics], Line, Tlen, _, _) ->
    yystate(29, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, [42|Ics], Line, Tlen, _, _) ->
    yystate(93, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(93, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,93};
yystate(92, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(92, [110|Ics], Line, Tlen, _, _) ->
    yystate(78, Ics, Line, Tlen+1, 21, Tlen);
yystate(92, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(92, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(92, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(92, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 109 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(92, [C|Ics], Line, Tlen, _, _) when C >= 111, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(92, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(92, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,92};
yystate(91, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(91, [47|Ics], Line, Tlen, _, _) ->
    yystate(42, Ics, Line, Tlen+1, 21, Tlen);
yystate(91, [42|Ics], Line, Tlen, _, _) ->
    yystate(106, Ics, Line, Tlen+1, 21, Tlen);
yystate(91, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(91, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(91, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(91, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(91, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(91, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,91};
yystate(90, [125|Ics], Line, Tlen, _, _) ->
    yystate(90, Ics, Line, Tlen+1, 11, Tlen);
yystate(90, [124|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 11, Tlen);
yystate(90, [123|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 11, Tlen);
yystate(90, [42|Ics], Line, Tlen, _, _) ->
    yystate(26, Ics, Line, Tlen+1, 11, Tlen);
yystate(90, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(75, Ics, Line, Tlen+1, 11, Tlen);
yystate(90, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(75, Ics, Line, Tlen+1, 11, Tlen);
yystate(90, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(75, Ics, Line, Tlen+1, 11, Tlen);
yystate(90, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(75, Ics, Line, Tlen+1, 11, Tlen);
yystate(90, Ics, Line, Tlen, _, _) ->
    {11,Tlen,Ics,Line,90};
yystate(89, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(89, [101|Ics], Line, Tlen, _, _) ->
    yystate(25, Ics, Line, Tlen+1, 21, Tlen);
yystate(89, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(89, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(89, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(89, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 100 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(89, [C|Ics], Line, Tlen, _, _) when C >= 102, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(89, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(89, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,89};
yystate(88, [123|Ics], Line, Tlen, _, _) ->
    yystate(95, Ics, Line, Tlen+1, 21, Tlen);
yystate(88, [42|Ics], Line, Tlen, _, _) ->
    yystate(31, Ics, Line, Tlen+1, 21, Tlen);
yystate(88, [32|Ics], Line, Tlen, _, _) ->
    yystate(88, Ics, Line, Tlen+1, 21, Tlen);
yystate(88, [9|Ics], Line, Tlen, _, _) ->
    yystate(88, Ics, Line, Tlen+1, 21, Tlen);
yystate(88, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(88, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(88, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(88, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(88, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(88, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,88};
yystate(87, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(87, [111|Ics], Line, Tlen, _, _) ->
    yystate(85, Ics, Line, Tlen+1, 21, Tlen);
yystate(87, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(87, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(87, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(87, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 110 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(87, [C|Ics], Line, Tlen, _, _) when C >= 112, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(87, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(87, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,87};
yystate(86, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(86, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(86, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(86, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(86, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(86, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(86, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,86};
yystate(85, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(85, [100|Ics], Line, Tlen, _, _) ->
    yystate(21, Ics, Line, Tlen+1, 21, Tlen);
yystate(85, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(85, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(85, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(85, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 99 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(85, [C|Ics], Line, Tlen, _, _) when C >= 101, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(85, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(85, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,85};
yystate(84, [123|Ics], Line, Tlen, _, _) ->
    yystate(95, Ics, Line, Tlen+1, 21, Tlen);
yystate(84, [42|Ics], Line, Tlen, _, _) ->
    yystate(31, Ics, Line, Tlen+1, 21, Tlen);
yystate(84, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(84, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(84, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(84, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(84, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,84};
yystate(83, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(83, [115|Ics], Line, Tlen, _, _) ->
    yystate(89, Ics, Line, Tlen+1, 21, Tlen);
yystate(83, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(83, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(83, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(83, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 114 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(83, [C|Ics], Line, Tlen, _, _) when C >= 116, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(83, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(83, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,83};
yystate(82, [123|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 21, Tlen);
yystate(82, [42|Ics], Line, Tlen, _, _) ->
    yystate(26, Ics, Line, Tlen+1, 21, Tlen);
yystate(82, [32|Ics], Line, Tlen, _, _) ->
    yystate(97, Ics, Line, Tlen+1, 21, Tlen);
yystate(82, [9|Ics], Line, Tlen, _, _) ->
    yystate(97, Ics, Line, Tlen+1, 21, Tlen);
yystate(82, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(75, Ics, Line, Tlen+1, 21, Tlen);
yystate(82, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(75, Ics, Line, Tlen+1, 21, Tlen);
yystate(82, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(75, Ics, Line, Tlen+1, 21, Tlen);
yystate(82, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(75, Ics, Line, Tlen+1, 21, Tlen);
yystate(82, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(75, Ics, Line, Tlen+1, 21, Tlen);
yystate(82, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,82};
yystate(81, [123|Ics], Line, Tlen, _, _) ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(81, [47|Ics], Line, Tlen, _, _) ->
    yystate(17, Ics, Line, Tlen+1, 3, Tlen);
yystate(81, [42|Ics], Line, Tlen, _, _) ->
    yystate(81, Ics, Line, Tlen+1, 3, Tlen);
yystate(81, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(81, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(81, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(81, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(81, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(81, Ics, Line, Tlen, _, _) ->
    {3,Tlen,Ics,Line,81};
yystate(80, [123|Ics], Line, Tlen, _, _) ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(80, [47|Ics], Line, Tlen, _, _) ->
    yystate(100, Ics, Line, Tlen+1, 7, Tlen);
yystate(80, [42|Ics], Line, Tlen, _, _) ->
    yystate(80, Ics, Line, Tlen+1, 7, Tlen);
yystate(80, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(80, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(80, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(80, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(80, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(80, Ics, Line, Tlen, _, _) ->
    {7,Tlen,Ics,Line,80};
yystate(79, [125|Ics], Line, Tlen, _, _) ->
    yystate(15, Ics, Line, Tlen+1, 9, Tlen);
yystate(79, [124|Ics], Line, Tlen, _, _) ->
    yystate(48, Ics, Line, Tlen+1, 9, Tlen);
yystate(79, [123|Ics], Line, Tlen, _, _) ->
    yystate(79, Ics, Line, Tlen+1, 9, Tlen);
yystate(79, [42|Ics], Line, Tlen, _, _) ->
    yystate(93, Ics, Line, Tlen+1, 9, Tlen);
yystate(79, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(48, Ics, Line, Tlen+1, 9, Tlen);
yystate(79, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(48, Ics, Line, Tlen+1, 9, Tlen);
yystate(79, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(48, Ics, Line, Tlen+1, 9, Tlen);
yystate(79, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(48, Ics, Line, Tlen+1, 9, Tlen);
yystate(79, Ics, Line, Tlen, _, _) ->
    {9,Tlen,Ics,Line,79};
yystate(78, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(78, [116|Ics], Line, Tlen, _, _) ->
    yystate(94, Ics, Line, Tlen+1, 21, Tlen);
yystate(78, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(78, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(78, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(78, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 115 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(78, [C|Ics], Line, Tlen, _, _) when C >= 117, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(78, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(78, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,78};
yystate(77, [123|Ics], Line, Tlen, _, _) ->
    yystate(95, Ics, Line, Tlen+1, 14, Tlen);
yystate(77, [42|Ics], Line, Tlen, _, _) ->
    yystate(31, Ics, Line, Tlen+1, 14, Tlen);
yystate(77, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(84, Ics, Line, Tlen+1, 14, Tlen);
yystate(77, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(84, Ics, Line, Tlen+1, 14, Tlen);
yystate(77, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(84, Ics, Line, Tlen+1, 14, Tlen);
yystate(77, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(84, Ics, Line, Tlen+1, 14, Tlen);
yystate(77, Ics, Line, Tlen, _, _) ->
    {14,Tlen,Ics,Line,77};
yystate(76, [123|Ics], Line, Tlen, _, _) ->
    yystate(76, Ics, Line, Tlen+1, 14, Tlen);
yystate(76, [42|Ics], Line, Tlen, _, _) ->
    yystate(96, Ics, Line, Tlen+1, 14, Tlen);
yystate(76, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(7, Ics, Line, Tlen+1, 14, Tlen);
yystate(76, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(7, Ics, Line, Tlen+1, 14, Tlen);
yystate(76, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(7, Ics, Line, Tlen+1, 14, Tlen);
yystate(76, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(7, Ics, Line, Tlen+1, 14, Tlen);
yystate(76, Ics, Line, Tlen, _, _) ->
    {14,Tlen,Ics,Line,76};
yystate(75, [125|Ics], Line, Tlen, _, _) ->
    yystate(90, Ics, Line, Tlen+1, 12, Tlen);
yystate(75, [124|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(75, [123|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(75, [42|Ics], Line, Tlen, _, _) ->
    yystate(26, Ics, Line, Tlen+1, 12, Tlen);
yystate(75, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(75, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(75, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(75, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(75, Ics, Line, Tlen, _, _) ->
    {12,Tlen,Ics,Line,75};
yystate(74, [123|Ics], Line, Tlen, _, _) ->
    yystate(34, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, [59|Ics], Line, Tlen, _, _) ->
    yystate(98, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, [47|Ics], Line, Tlen, _, _) ->
    yystate(14, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, [42|Ics], Line, Tlen, _, _) ->
    yystate(74, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 58 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, [C|Ics], Line, Tlen, _, _) when C >= 60, C =< 122 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(74, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,74};
yystate(73, [125|Ics], Line, Tlen, _, _) ->
    yystate(99, Ics, Line, Tlen+1, 12, Tlen);
yystate(73, [124|Ics], Line, Tlen, _, _) ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(73, [123|Ics], Line, Tlen, _, _) ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(73, [42|Ics], Line, Tlen, _, _) ->
    yystate(35, Ics, Line, Tlen+1, 12, Tlen);
yystate(73, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(73, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(73, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(73, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(73, Ics, Line, Tlen, _, _) ->
    {12,Tlen,Ics,Line,73};
yystate(72, [123|Ics], Line, Tlen, _, _) ->
    yystate(103, Ics, Line, Tlen+1, 2, Tlen);
yystate(72, [42|Ics], Line, Tlen, _, _) ->
    yystate(24, Ics, Line, Tlen+1, 2, Tlen);
yystate(72, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(44, Ics, Line, Tlen+1, 2, Tlen);
yystate(72, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(44, Ics, Line, Tlen+1, 2, Tlen);
yystate(72, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(44, Ics, Line, Tlen+1, 2, Tlen);
yystate(72, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(44, Ics, Line, Tlen+1, 2, Tlen);
yystate(72, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,72};
yystate(71, [125|Ics], Line, Tlen, _, _) ->
    yystate(5, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [124|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [117|Ics], Line, Tlen, _, _) ->
    yystate(83, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [116|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [115|Ics], Line, Tlen, _, _) ->
    yystate(113, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [113|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [114|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [112|Ics], Line, Tlen, _, _) ->
    yystate(20, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [110|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [111|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [109|Ics], Line, Tlen, _, _) ->
    yystate(87, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [105|Ics], Line, Tlen, _, _) ->
    yystate(112, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [103|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [104|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [102|Ics], Line, Tlen, _, _) ->
    yystate(41, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [101|Ics], Line, Tlen, _, _) ->
    yystate(92, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [100|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [99|Ics], Line, Tlen, _, _) ->
    yystate(61, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [47|Ics], Line, Tlen, _, _) ->
    yystate(91, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [35|Ics], Line, Tlen, _, _) ->
    yystate(12, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [33|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [34|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [32|Ics], Line, Tlen, _, _) ->
    yystate(71, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [9|Ics], Line, Tlen, _, _) ->
    yystate(22, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [C|Ics], Line, Tlen, _, _) when C >= 36, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 98 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [C|Ics], Line, Tlen, _, _) when C >= 106, C =< 108 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [C|Ics], Line, Tlen, _, _) when C >= 118, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(71, Ics, Line, Tlen, _, _) ->
    {20,Tlen,Ics,Line,71};
yystate(70, [123|Ics], Line, Tlen, _, _) ->
    yystate(10, Ics, Line, Tlen+1, 21, Tlen);
yystate(70, [42|Ics], Line, Tlen, _, _) ->
    yystate(117, Ics, Line, Tlen+1, 21, Tlen);
yystate(70, [32|Ics], Line, Tlen, _, _) ->
    yystate(8, Ics, Line, Tlen+1, 21, Tlen);
yystate(70, [9|Ics], Line, Tlen, _, _) ->
    yystate(8, Ics, Line, Tlen+1, 21, Tlen);
yystate(70, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(70, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(70, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(70, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(70, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(70, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,70};
yystate(69, [123|Ics], Line, Tlen, _, _) ->
    yystate(103, Ics, Line, Tlen+1, 14, Tlen);
yystate(69, [42|Ics], Line, Tlen, _, _) ->
    yystate(24, Ics, Line, Tlen+1, 14, Tlen);
yystate(69, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(44, Ics, Line, Tlen+1, 14, Tlen);
yystate(69, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(44, Ics, Line, Tlen+1, 14, Tlen);
yystate(69, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(44, Ics, Line, Tlen+1, 14, Tlen);
yystate(69, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(44, Ics, Line, Tlen+1, 14, Tlen);
yystate(69, Ics, Line, Tlen, _, _) ->
    {14,Tlen,Ics,Line,69};
yystate(68, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(68, [99|Ics], Line, Tlen, _, _) ->
    yystate(104, Ics, Line, Tlen+1, 21, Tlen);
yystate(68, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(68, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(68, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(68, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 98 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(68, [C|Ics], Line, Tlen, _, _) when C >= 100, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(68, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(68, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,68};
yystate(67, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(67, [110|Ics], Line, Tlen, _, _) ->
    yystate(105, Ics, Line, Tlen+1, 21, Tlen);
yystate(67, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(67, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(67, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(67, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 109 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(67, [C|Ics], Line, Tlen, _, _) when C >= 111, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(67, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(67, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,67};
yystate(66, [123|Ics], Line, Tlen, _, _) ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(66, [47|Ics], Line, Tlen, _, _) ->
    yystate(6, Ics, Line, Tlen+1, 1, Tlen);
yystate(66, [42|Ics], Line, Tlen, _, _) ->
    yystate(66, Ics, Line, Tlen+1, 1, Tlen);
yystate(66, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(66, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(66, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(66, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(66, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(66, Ics, Line, Tlen, _, _) ->
    {1,Tlen,Ics,Line,66};
yystate(65, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(65, [105|Ics], Line, Tlen, _, _) ->
    yystate(1, Ics, Line, Tlen+1, 21, Tlen);
yystate(65, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(65, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(65, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(65, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 104 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(65, [C|Ics], Line, Tlen, _, _) when C >= 106, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(65, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(65, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,65};
yystate(64, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(64, [100|Ics], Line, Tlen, _, _) ->
    yystate(0, Ics, Line, Tlen+1, 21, Tlen);
yystate(64, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(64, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(64, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(64, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 99 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(64, [C|Ics], Line, Tlen, _, _) when C >= 101, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(64, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(64, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,64};
yystate(63, [123|Ics], Line, Tlen, _, _) ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(63, [47|Ics], Line, Tlen, _, _) ->
    yystate(109, Ics, Line, Tlen+1, 8, Tlen);
yystate(63, [42|Ics], Line, Tlen, _, _) ->
    yystate(63, Ics, Line, Tlen+1, 8, Tlen);
yystate(63, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(63, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(63, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(63, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(63, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(63, Ics, Line, Tlen, _, _) ->
    {8,Tlen,Ics,Line,63};
yystate(62, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(62, [105|Ics], Line, Tlen, _, _) ->
    yystate(110, Ics, Line, Tlen+1, 21, Tlen);
yystate(62, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(62, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(62, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(62, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 104 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(62, [C|Ics], Line, Tlen, _, _) when C >= 106, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(62, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(62, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,62};
yystate(61, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(61, [111|Ics], Line, Tlen, _, _) ->
    yystate(111, Ics, Line, Tlen+1, 21, Tlen);
yystate(61, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(61, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(61, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(61, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 110 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(61, [C|Ics], Line, Tlen, _, _) when C >= 112, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(61, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(61, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,61};
yystate(60, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(60, [108|Ics], Line, Tlen, _, _) ->
    yystate(3, Ics, Line, Tlen+1, 21, Tlen);
yystate(60, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(60, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(60, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(60, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 107 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(60, [C|Ics], Line, Tlen, _, _) when C >= 109, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(60, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(60, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,60};
yystate(59, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(59, [114|Ics], Line, Tlen, _, _) ->
    yystate(4, Ics, Line, Tlen+1, 21, Tlen);
yystate(59, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(59, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(59, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(59, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 113 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(59, [C|Ics], Line, Tlen, _, _) when C >= 115, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(59, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(59, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,59};
yystate(58, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [102|Ics], Line, Tlen, _, _) ->
    yystate(114, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [101|Ics], Line, Tlen, _, _) ->
    yystate(92, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [32|Ics], Line, Tlen, _, _) ->
    yystate(58, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [9|Ics], Line, Tlen, _, _) ->
    yystate(58, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 100 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [C|Ics], Line, Tlen, _, _) when C >= 103, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(58, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,58};
yystate(57, [123|Ics], Line, Tlen, _, _) ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(57, [42|Ics], Line, Tlen, _, _) ->
    yystate(66, Ics, Line, Tlen+1, 1, Tlen);
yystate(57, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(57, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(57, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(57, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(57, Ics, Line, Tlen+1, 1, Tlen);
yystate(57, Ics, Line, Tlen, _, _) ->
    {1,Tlen,Ics,Line,57};
yystate(56, [123|Ics], Line, Tlen, _, _) ->
    yystate(53, Ics, Line, Tlen+1, 21, Tlen);
yystate(56, [42|Ics], Line, Tlen, _, _) ->
    yystate(117, Ics, Line, Tlen+1, 21, Tlen);
yystate(56, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(56, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(56, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(56, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(56, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,56};
yystate(55, [123|Ics], Line, Tlen, _, _) ->
    yystate(53, Ics, Line, Tlen+1, 2, Tlen);
yystate(55, [42|Ics], Line, Tlen, _, _) ->
    yystate(117, Ics, Line, Tlen+1, 2, Tlen);
yystate(55, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(56, Ics, Line, Tlen+1, 2, Tlen);
yystate(55, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(56, Ics, Line, Tlen+1, 2, Tlen);
yystate(55, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(56, Ics, Line, Tlen+1, 2, Tlen);
yystate(55, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(56, Ics, Line, Tlen+1, 2, Tlen);
yystate(55, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,55};
yystate(54, [125|Ics], Line, Tlen, _, _) ->
    yystate(5, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [124|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [117|Ics], Line, Tlen, _, _) ->
    yystate(83, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [116|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [115|Ics], Line, Tlen, _, _) ->
    yystate(113, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [113|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [114|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [112|Ics], Line, Tlen, _, _) ->
    yystate(20, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [110|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [111|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [109|Ics], Line, Tlen, _, _) ->
    yystate(87, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [105|Ics], Line, Tlen, _, _) ->
    yystate(112, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [103|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [104|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [102|Ics], Line, Tlen, _, _) ->
    yystate(41, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [101|Ics], Line, Tlen, _, _) ->
    yystate(92, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [100|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [99|Ics], Line, Tlen, _, _) ->
    yystate(61, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [47|Ics], Line, Tlen, _, _) ->
    yystate(91, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [35|Ics], Line, Tlen, _, _) ->
    yystate(12, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [33|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [34|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [32|Ics], Line, Tlen, _, _) ->
    yystate(71, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [13|Ics], Line, Tlen, _, _) ->
    yystate(101, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [11|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [12|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [10|Ics], Line, Tlen, _, _) ->
    yystate(37, Ics, Line+1, Tlen+1, 20, Tlen);
yystate(54, [9|Ics], Line, Tlen, _, _) ->
    yystate(22, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [C|Ics], Line, Tlen, _, _) when C >= 14, C =< 31 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [C|Ics], Line, Tlen, _, _) when C >= 36, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 98 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [C|Ics], Line, Tlen, _, _) when C >= 106, C =< 108 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [C|Ics], Line, Tlen, _, _) when C >= 118, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(54, Ics, Line, Tlen, _, _) ->
    {20,Tlen,Ics,Line,54};
yystate(53, [123|Ics], Line, Tlen, _, _) ->
    yystate(53, Ics, Line, Tlen+1, 4, Tlen);
yystate(53, [42|Ics], Line, Tlen, _, _) ->
    yystate(117, Ics, Line, Tlen+1, 4, Tlen);
yystate(53, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(56, Ics, Line, Tlen+1, 4, Tlen);
yystate(53, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(56, Ics, Line, Tlen+1, 4, Tlen);
yystate(53, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(56, Ics, Line, Tlen+1, 4, Tlen);
yystate(53, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(56, Ics, Line, Tlen+1, 4, Tlen);
yystate(53, Ics, Line, Tlen, _, _) ->
    {4,Tlen,Ics,Line,53};
yystate(52, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(52, [101|Ics], Line, Tlen, _, _) ->
    yystate(116, Ics, Line, Tlen+1, 21, Tlen);
yystate(52, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(52, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(52, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(52, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 100 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(52, [C|Ics], Line, Tlen, _, _) when C >= 102, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(52, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(52, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,52};
yystate(51, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 2, Tlen);
yystate(51, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 2, Tlen);
yystate(51, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 2, Tlen);
yystate(51, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 2, Tlen);
yystate(51, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 2, Tlen);
yystate(51, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 2, Tlen);
yystate(51, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,51};
yystate(50, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(50, [110|Ics], Line, Tlen, _, _) ->
    yystate(9, Ics, Line, Tlen+1, 21, Tlen);
yystate(50, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(50, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(50, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(50, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 109 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(50, [C|Ics], Line, Tlen, _, _) when C >= 111, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(50, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(50, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,50};
yystate(49, [123|Ics], Line, Tlen, _, _) ->
    yystate(34, Ics, Line, Tlen+1, 21, Tlen);
yystate(49, [59|Ics], Line, Tlen, _, _) ->
    yystate(98, Ics, Line, Tlen+1, 21, Tlen);
yystate(49, [42|Ics], Line, Tlen, _, _) ->
    yystate(74, Ics, Line, Tlen+1, 21, Tlen);
yystate(49, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(49, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(49, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 58 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(49, [C|Ics], Line, Tlen, _, _) when C >= 60, C =< 122 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(49, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(49, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,49};
yystate(48, [125|Ics], Line, Tlen, _, _) ->
    yystate(15, Ics, Line, Tlen+1, 21, Tlen);
yystate(48, [124|Ics], Line, Tlen, _, _) ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(48, [123|Ics], Line, Tlen, _, _) ->
    yystate(79, Ics, Line, Tlen+1, 21, Tlen);
yystate(48, [42|Ics], Line, Tlen, _, _) ->
    yystate(93, Ics, Line, Tlen+1, 21, Tlen);
yystate(48, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(48, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(48, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(48, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(48, Ics, Line, Tlen+1, 21, Tlen);
yystate(48, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,48};
yystate(47, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(47, [115|Ics], Line, Tlen, _, _) ->
    yystate(16, Ics, Line, Tlen+1, 21, Tlen);
yystate(47, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(47, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(47, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(47, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 114 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(47, [C|Ics], Line, Tlen, _, _) when C >= 116, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(47, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(47, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,47};
yystate(46, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(46, [116|Ics], Line, Tlen, _, _) ->
    yystate(13, Ics, Line, Tlen+1, 21, Tlen);
yystate(46, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(46, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(46, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(46, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 115 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(46, [C|Ics], Line, Tlen, _, _) when C >= 117, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(46, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(46, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,46};
yystate(45, [123|Ics], Line, Tlen, _, _) ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(45, [42|Ics], Line, Tlen, _, _) ->
    yystate(63, Ics, Line, Tlen+1, 8, Tlen);
yystate(45, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(45, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(45, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(45, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(45, Ics, Line, Tlen+1, 8, Tlen);
yystate(45, Ics, Line, Tlen, _, _) ->
    {8,Tlen,Ics,Line,45};
yystate(44, [123|Ics], Line, Tlen, _, _) ->
    yystate(103, Ics, Line, Tlen+1, 21, Tlen);
yystate(44, [42|Ics], Line, Tlen, _, _) ->
    yystate(24, Ics, Line, Tlen+1, 21, Tlen);
yystate(44, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(44, Ics, Line, Tlen+1, 21, Tlen);
yystate(44, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(44, Ics, Line, Tlen+1, 21, Tlen);
yystate(44, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(44, Ics, Line, Tlen+1, 21, Tlen);
yystate(44, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(44, Ics, Line, Tlen+1, 21, Tlen);
yystate(44, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,44};
yystate(43, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(43, [98|Ics], Line, Tlen, _, _) ->
    yystate(107, Ics, Line, Tlen+1, 21, Tlen);
yystate(43, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(43, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(43, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(43, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 97 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(43, [C|Ics], Line, Tlen, _, _) when C >= 99, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(43, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(43, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,43};
yystate(42, [123|Ics], Line, Tlen, _, _) ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(42, [42|Ics], Line, Tlen, _, _) ->
    yystate(81, Ics, Line, Tlen+1, 3, Tlen);
yystate(42, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(42, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(42, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(42, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(42, Ics, Line, Tlen+1, 3, Tlen);
yystate(42, Ics, Line, Tlen, _, _) ->
    {3,Tlen,Ics,Line,42};
yystate(41, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [117|Ics], Line, Tlen, _, _) ->
    yystate(18, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [115|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [116|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [114|Ics], Line, Tlen, _, _) ->
    yystate(11, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 113 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [C|Ics], Line, Tlen, _, _) when C >= 118, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(41, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,41};
yystate(40, [123|Ics], Line, Tlen, _, _) ->
    yystate(76, Ics, Line, Tlen+1, 2, Tlen);
yystate(40, [42|Ics], Line, Tlen, _, _) ->
    yystate(96, Ics, Line, Tlen+1, 2, Tlen);
yystate(40, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(7, Ics, Line, Tlen+1, 2, Tlen);
yystate(40, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(7, Ics, Line, Tlen+1, 2, Tlen);
yystate(40, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(7, Ics, Line, Tlen+1, 2, Tlen);
yystate(40, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(7, Ics, Line, Tlen+1, 2, Tlen);
yystate(40, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,40};
yystate(39, [123|Ics], Line, Tlen, _, _) ->
    yystate(103, Ics, Line, Tlen+1, 17, Tlen);
yystate(39, [42|Ics], Line, Tlen, _, _) ->
    yystate(24, Ics, Line, Tlen+1, 17, Tlen);
yystate(39, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(44, Ics, Line, Tlen+1, 17, Tlen);
yystate(39, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(44, Ics, Line, Tlen+1, 17, Tlen);
yystate(39, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(44, Ics, Line, Tlen+1, 17, Tlen);
yystate(39, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(44, Ics, Line, Tlen+1, 17, Tlen);
yystate(39, Ics, Line, Tlen, _, _) ->
    {17,Tlen,Ics,Line,39};
yystate(38, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(38, [108|Ics], Line, Tlen, _, _) ->
    yystate(102, Ics, Line, Tlen+1, 21, Tlen);
yystate(38, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(38, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(38, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(38, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 107 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(38, [C|Ics], Line, Tlen, _, _) when C >= 109, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(38, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(38, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,38};
yystate(37, [13|Ics], Line, Tlen, _, _) ->
    yystate(37, Ics, Line, Tlen+1, 19, Tlen);
yystate(37, [10|Ics], Line, Tlen, _, _) ->
    yystate(37, Ics, Line+1, Tlen+1, 19, Tlen);
yystate(37, Ics, Line, Tlen, _, _) ->
    {19,Tlen,Ics,Line,37};
yystate(36, [123|Ics], Line, Tlen, _, _) ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(36, [42|Ics], Line, Tlen, _, _) ->
    yystate(80, Ics, Line, Tlen+1, 7, Tlen);
yystate(36, [32|Ics], Line, Tlen, _, _) ->
    yystate(36, Ics, Line, Tlen+1, 7, Tlen);
yystate(36, [9|Ics], Line, Tlen, _, _) ->
    yystate(36, Ics, Line, Tlen+1, 7, Tlen);
yystate(36, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(36, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(36, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(36, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(36, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(36, Ics, Line, Tlen, _, _) ->
    {7,Tlen,Ics,Line,36};
yystate(35, [125|Ics], Line, Tlen, _, _) ->
    yystate(99, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, [124|Ics], Line, Tlen, _, _) ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, [123|Ics], Line, Tlen, _, _) ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, [47|Ics], Line, Tlen, _, _) ->
    yystate(28, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, [42|Ics], Line, Tlen, _, _) ->
    yystate(35, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(73, Ics, Line, Tlen+1, 12, Tlen);
yystate(35, Ics, Line, Tlen, _, _) ->
    {12,Tlen,Ics,Line,35};
yystate(34, [123|Ics], Line, Tlen, _, _) ->
    yystate(34, Ics, Line, Tlen+1, 14, Tlen);
yystate(34, [59|Ics], Line, Tlen, _, _) ->
    yystate(98, Ics, Line, Tlen+1, 14, Tlen);
yystate(34, [42|Ics], Line, Tlen, _, _) ->
    yystate(74, Ics, Line, Tlen+1, 14, Tlen);
yystate(34, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(49, Ics, Line, Tlen+1, 14, Tlen);
yystate(34, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(49, Ics, Line, Tlen+1, 14, Tlen);
yystate(34, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 58 ->
    yystate(49, Ics, Line, Tlen+1, 14, Tlen);
yystate(34, [C|Ics], Line, Tlen, _, _) when C >= 60, C =< 122 ->
    yystate(49, Ics, Line, Tlen+1, 14, Tlen);
yystate(34, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(49, Ics, Line, Tlen+1, 14, Tlen);
yystate(34, Ics, Line, Tlen, _, _) ->
    {14,Tlen,Ics,Line,34};
yystate(33, [125|Ics], Line, Tlen, _, _) ->
    yystate(90, Ics, Line, Tlen+1, 2, Tlen);
yystate(33, [124|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 2, Tlen);
yystate(33, [123|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 2, Tlen);
yystate(33, [42|Ics], Line, Tlen, _, _) ->
    yystate(26, Ics, Line, Tlen+1, 2, Tlen);
yystate(33, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(75, Ics, Line, Tlen+1, 2, Tlen);
yystate(33, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(75, Ics, Line, Tlen+1, 2, Tlen);
yystate(33, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(75, Ics, Line, Tlen+1, 2, Tlen);
yystate(33, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(75, Ics, Line, Tlen+1, 2, Tlen);
yystate(33, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,33};
yystate(32, [123|Ics], Line, Tlen, _, _) ->
    yystate(95, Ics, Line, Tlen+1, 2, Tlen);
yystate(32, [42|Ics], Line, Tlen, _, _) ->
    yystate(31, Ics, Line, Tlen+1, 2, Tlen);
yystate(32, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(84, Ics, Line, Tlen+1, 2, Tlen);
yystate(32, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(84, Ics, Line, Tlen+1, 2, Tlen);
yystate(32, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(84, Ics, Line, Tlen+1, 2, Tlen);
yystate(32, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(84, Ics, Line, Tlen+1, 2, Tlen);
yystate(32, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,32};
yystate(31, [123|Ics], Line, Tlen, _, _) ->
    yystate(95, Ics, Line, Tlen+1, 21, Tlen);
yystate(31, [47|Ics], Line, Tlen, _, _) ->
    yystate(32, Ics, Line, Tlen+1, 21, Tlen);
yystate(31, [42|Ics], Line, Tlen, _, _) ->
    yystate(31, Ics, Line, Tlen+1, 21, Tlen);
yystate(31, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(31, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(31, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(31, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(31, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(31, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,31};
yystate(30, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(30, [114|Ics], Line, Tlen, _, _) ->
    yystate(62, Ics, Line, Tlen+1, 21, Tlen);
yystate(30, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(30, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(30, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(30, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 113 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(30, [C|Ics], Line, Tlen, _, _) when C >= 115, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(30, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(30, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,30};
yystate(29, [125|Ics], Line, Tlen, _, _) ->
    yystate(15, Ics, Line, Tlen+1, 2, Tlen);
yystate(29, [124|Ics], Line, Tlen, _, _) ->
    yystate(48, Ics, Line, Tlen+1, 2, Tlen);
yystate(29, [123|Ics], Line, Tlen, _, _) ->
    yystate(79, Ics, Line, Tlen+1, 2, Tlen);
yystate(29, [42|Ics], Line, Tlen, _, _) ->
    yystate(93, Ics, Line, Tlen+1, 2, Tlen);
yystate(29, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(48, Ics, Line, Tlen+1, 2, Tlen);
yystate(29, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(48, Ics, Line, Tlen+1, 2, Tlen);
yystate(29, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(48, Ics, Line, Tlen+1, 2, Tlen);
yystate(29, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(48, Ics, Line, Tlen+1, 2, Tlen);
yystate(29, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,29};
yystate(28, [125|Ics], Line, Tlen, _, _) ->
    yystate(99, Ics, Line, Tlen+1, 2, Tlen);
yystate(28, [124|Ics], Line, Tlen, _, _) ->
    yystate(73, Ics, Line, Tlen+1, 2, Tlen);
yystate(28, [123|Ics], Line, Tlen, _, _) ->
    yystate(73, Ics, Line, Tlen+1, 2, Tlen);
yystate(28, [42|Ics], Line, Tlen, _, _) ->
    yystate(35, Ics, Line, Tlen+1, 2, Tlen);
yystate(28, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(73, Ics, Line, Tlen+1, 2, Tlen);
yystate(28, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(73, Ics, Line, Tlen+1, 2, Tlen);
yystate(28, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(73, Ics, Line, Tlen+1, 2, Tlen);
yystate(28, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(73, Ics, Line, Tlen+1, 2, Tlen);
yystate(28, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,28};
yystate(27, [123|Ics], Line, Tlen, _, _) ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(27, [42|Ics], Line, Tlen, _, _) ->
    yystate(80, Ics, Line, Tlen+1, 7, Tlen);
yystate(27, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(27, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(27, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(27, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(27, Ics, Line, Tlen+1, 7, Tlen);
yystate(27, Ics, Line, Tlen, _, _) ->
    {7,Tlen,Ics,Line,27};
yystate(26, [125|Ics], Line, Tlen, _, _) ->
    yystate(90, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, [124|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, [123|Ics], Line, Tlen, _, _) ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, [47|Ics], Line, Tlen, _, _) ->
    yystate(33, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, [42|Ics], Line, Tlen, _, _) ->
    yystate(26, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(75, Ics, Line, Tlen+1, 12, Tlen);
yystate(26, Ics, Line, Tlen, _, _) ->
    {12,Tlen,Ics,Line,26};
yystate(25, [123|Ics], Line, Tlen, _, _) ->
    yystate(34, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [59|Ics], Line, Tlen, _, _) ->
    yystate(98, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [42|Ics], Line, Tlen, _, _) ->
    yystate(74, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [32|Ics], Line, Tlen, _, _) ->
    yystate(25, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [9|Ics], Line, Tlen, _, _) ->
    yystate(25, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 58 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [C|Ics], Line, Tlen, _, _) when C >= 60, C =< 122 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(49, Ics, Line, Tlen+1, 21, Tlen);
yystate(25, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,25};
yystate(24, [123|Ics], Line, Tlen, _, _) ->
    yystate(103, Ics, Line, Tlen+1, 21, Tlen);
yystate(24, [47|Ics], Line, Tlen, _, _) ->
    yystate(72, Ics, Line, Tlen+1, 21, Tlen);
yystate(24, [42|Ics], Line, Tlen, _, _) ->
    yystate(24, Ics, Line, Tlen+1, 21, Tlen);
yystate(24, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(44, Ics, Line, Tlen+1, 21, Tlen);
yystate(24, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(44, Ics, Line, Tlen+1, 21, Tlen);
yystate(24, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(44, Ics, Line, Tlen+1, 21, Tlen);
yystate(24, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 122 ->
    yystate(44, Ics, Line, Tlen+1, 21, Tlen);
yystate(24, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(44, Ics, Line, Tlen+1, 21, Tlen);
yystate(24, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,24};
yystate(23, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(23, [122|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(23, [121|Ics], Line, Tlen, _, _) ->
    yystate(2, Ics, Line, Tlen+1, 21, Tlen);
yystate(23, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(23, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(23, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(23, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 120 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(23, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(23, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,23};
yystate(22, [125|Ics], Line, Tlen, _, _) ->
    yystate(5, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [124|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [117|Ics], Line, Tlen, _, _) ->
    yystate(83, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [116|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [115|Ics], Line, Tlen, _, _) ->
    yystate(113, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [113|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [114|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [112|Ics], Line, Tlen, _, _) ->
    yystate(20, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [110|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [111|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [109|Ics], Line, Tlen, _, _) ->
    yystate(87, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [105|Ics], Line, Tlen, _, _) ->
    yystate(112, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [103|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [104|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [102|Ics], Line, Tlen, _, _) ->
    yystate(41, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [101|Ics], Line, Tlen, _, _) ->
    yystate(92, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [100|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [99|Ics], Line, Tlen, _, _) ->
    yystate(61, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [47|Ics], Line, Tlen, _, _) ->
    yystate(91, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [35|Ics], Line, Tlen, _, _) ->
    yystate(12, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [33|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [34|Ics], Line, Tlen, _, _) ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [32|Ics], Line, Tlen, _, _) ->
    yystate(22, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [9|Ics], Line, Tlen, _, _) ->
    yystate(22, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [C|Ics], Line, Tlen, _, _) when C >= 36, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 46 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [C|Ics], Line, Tlen, _, _) when C >= 48, C =< 98 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [C|Ics], Line, Tlen, _, _) when C >= 106, C =< 108 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [C|Ics], Line, Tlen, _, _) when C >= 118, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(86, Ics, Line, Tlen+1, 20, Tlen);
yystate(22, Ics, Line, Tlen, _, _) ->
    {20,Tlen,Ics,Line,22};
yystate(21, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(21, [117|Ics], Line, Tlen, _, _) ->
    yystate(38, Ics, Line, Tlen+1, 21, Tlen);
yystate(21, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(21, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(21, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(21, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 116 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(21, [C|Ics], Line, Tlen, _, _) when C >= 118, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(21, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(21, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,21};
yystate(20, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(20, [117|Ics], Line, Tlen, _, _) ->
    yystate(43, Ics, Line, Tlen+1, 21, Tlen);
yystate(20, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(20, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(20, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(20, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 116 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(20, [C|Ics], Line, Tlen, _, _) when C >= 118, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(20, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(20, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,20};
yystate(19, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 14, Tlen);
yystate(19, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 14, Tlen);
yystate(19, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 14, Tlen);
yystate(19, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 14, Tlen);
yystate(19, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 14, Tlen);
yystate(19, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 14, Tlen);
yystate(19, Ics, Line, Tlen, _, _) ->
    {14,Tlen,Ics,Line,19};
yystate(18, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(18, [110|Ics], Line, Tlen, _, _) ->
    yystate(82, Ics, Line, Tlen+1, 21, Tlen);
yystate(18, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(18, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(18, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(18, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 109 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(18, [C|Ics], Line, Tlen, _, _) when C >= 111, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(18, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(18, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,18};
yystate(17, [123|Ics], Line, Tlen, _, _) ->
    yystate(42, Ics, Line, Tlen+1, 2, Tlen);
yystate(17, [42|Ics], Line, Tlen, _, _) ->
    yystate(81, Ics, Line, Tlen+1, 2, Tlen);
yystate(17, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(42, Ics, Line, Tlen+1, 2, Tlen);
yystate(17, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(42, Ics, Line, Tlen+1, 2, Tlen);
yystate(17, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(42, Ics, Line, Tlen+1, 2, Tlen);
yystate(17, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(42, Ics, Line, Tlen+1, 2, Tlen);
yystate(17, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,17};
yystate(16, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(16, [116|Ics], Line, Tlen, _, _) ->
    yystate(36, Ics, Line, Tlen+1, 21, Tlen);
yystate(16, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(16, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(16, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(16, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 115 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(16, [C|Ics], Line, Tlen, _, _) when C >= 117, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(16, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(16, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,16};
yystate(15, [125|Ics], Line, Tlen, _, _) ->
    yystate(15, Ics, Line, Tlen+1, 10, Tlen);
yystate(15, [124|Ics], Line, Tlen, _, _) ->
    yystate(48, Ics, Line, Tlen+1, 10, Tlen);
yystate(15, [123|Ics], Line, Tlen, _, _) ->
    yystate(79, Ics, Line, Tlen+1, 10, Tlen);
yystate(15, [42|Ics], Line, Tlen, _, _) ->
    yystate(93, Ics, Line, Tlen+1, 10, Tlen);
yystate(15, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(48, Ics, Line, Tlen+1, 10, Tlen);
yystate(15, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(48, Ics, Line, Tlen+1, 10, Tlen);
yystate(15, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(48, Ics, Line, Tlen+1, 10, Tlen);
yystate(15, [C|Ics], Line, Tlen, _, _) when C >= 126 ->
    yystate(48, Ics, Line, Tlen+1, 10, Tlen);
yystate(15, Ics, Line, Tlen, _, _) ->
    {10,Tlen,Ics,Line,15};
yystate(14, [123|Ics], Line, Tlen, _, _) ->
    yystate(34, Ics, Line, Tlen+1, 2, Tlen);
yystate(14, [59|Ics], Line, Tlen, _, _) ->
    yystate(98, Ics, Line, Tlen+1, 2, Tlen);
yystate(14, [42|Ics], Line, Tlen, _, _) ->
    yystate(74, Ics, Line, Tlen+1, 2, Tlen);
yystate(14, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(49, Ics, Line, Tlen+1, 2, Tlen);
yystate(14, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(49, Ics, Line, Tlen+1, 2, Tlen);
yystate(14, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 58 ->
    yystate(49, Ics, Line, Tlen+1, 2, Tlen);
yystate(14, [C|Ics], Line, Tlen, _, _) when C >= 60, C =< 122 ->
    yystate(49, Ics, Line, Tlen+1, 2, Tlen);
yystate(14, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(49, Ics, Line, Tlen+1, 2, Tlen);
yystate(14, Ics, Line, Tlen, _, _) ->
    {2,Tlen,Ics,Line,14};
yystate(13, [123|Ics], Line, Tlen, _, _) ->
    yystate(77, Ics, Line, Tlen+1, 21, Tlen);
yystate(13, [42|Ics], Line, Tlen, _, _) ->
    yystate(31, Ics, Line, Tlen+1, 21, Tlen);
yystate(13, [32|Ics], Line, Tlen, _, _) ->
    yystate(88, Ics, Line, Tlen+1, 21, Tlen);
yystate(13, [9|Ics], Line, Tlen, _, _) ->
    yystate(88, Ics, Line, Tlen+1, 21, Tlen);
yystate(13, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(13, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(13, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(13, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(13, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(84, Ics, Line, Tlen+1, 21, Tlen);
yystate(13, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,13};
yystate(12, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(12, [91|Ics], Line, Tlen, _, _) ->
    yystate(7, Ics, Line, Tlen+1, 21, Tlen);
yystate(12, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(12, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(12, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(12, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 90 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(12, [C|Ics], Line, Tlen, _, _) when C >= 92, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(12, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(12, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,12};
yystate(11, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(11, [105|Ics], Line, Tlen, _, _) ->
    yystate(52, Ics, Line, Tlen+1, 21, Tlen);
yystate(11, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(11, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(11, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(11, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 104 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(11, [C|Ics], Line, Tlen, _, _) when C >= 106, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(11, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(11, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,11};
yystate(10, [123|Ics], Line, Tlen, _, _) ->
    yystate(53, Ics, Line, Tlen+1, 14, Tlen);
yystate(10, [42|Ics], Line, Tlen, _, _) ->
    yystate(117, Ics, Line, Tlen+1, 14, Tlen);
yystate(10, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(56, Ics, Line, Tlen+1, 14, Tlen);
yystate(10, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(56, Ics, Line, Tlen+1, 14, Tlen);
yystate(10, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(56, Ics, Line, Tlen+1, 14, Tlen);
yystate(10, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(56, Ics, Line, Tlen+1, 14, Tlen);
yystate(10, Ics, Line, Tlen, _, _) ->
    {14,Tlen,Ics,Line,10};
yystate(9, [123|Ics], Line, Tlen, _, _) ->
    yystate(73, Ics, Line, Tlen+1, 21, Tlen);
yystate(9, [42|Ics], Line, Tlen, _, _) ->
    yystate(35, Ics, Line, Tlen+1, 21, Tlen);
yystate(9, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(73, Ics, Line, Tlen+1, 21, Tlen);
yystate(9, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(73, Ics, Line, Tlen+1, 21, Tlen);
yystate(9, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(73, Ics, Line, Tlen+1, 21, Tlen);
yystate(9, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(73, Ics, Line, Tlen+1, 21, Tlen);
yystate(9, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,9};
yystate(8, [123|Ics], Line, Tlen, _, _) ->
    yystate(53, Ics, Line, Tlen+1, 21, Tlen);
yystate(8, [42|Ics], Line, Tlen, _, _) ->
    yystate(117, Ics, Line, Tlen+1, 21, Tlen);
yystate(8, [32|Ics], Line, Tlen, _, _) ->
    yystate(8, Ics, Line, Tlen+1, 21, Tlen);
yystate(8, [9|Ics], Line, Tlen, _, _) ->
    yystate(8, Ics, Line, Tlen+1, 21, Tlen);
yystate(8, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(8, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(8, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(8, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(8, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(56, Ics, Line, Tlen+1, 21, Tlen);
yystate(8, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,8};
yystate(7, [123|Ics], Line, Tlen, _, _) ->
    yystate(76, Ics, Line, Tlen+1, 18, Tlen);
yystate(7, [42|Ics], Line, Tlen, _, _) ->
    yystate(96, Ics, Line, Tlen+1, 18, Tlen);
yystate(7, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(7, Ics, Line, Tlen+1, 18, Tlen);
yystate(7, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(7, Ics, Line, Tlen+1, 18, Tlen);
yystate(7, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(7, Ics, Line, Tlen+1, 18, Tlen);
yystate(7, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(7, Ics, Line, Tlen+1, 18, Tlen);
yystate(7, Ics, Line, Tlen, _, _) ->
    {18,Tlen,Ics,Line,7};
yystate(6, [123|Ics], Line, Tlen, _, _) ->
    yystate(57, Ics, Line, Tlen+1, 0, Tlen);
yystate(6, [42|Ics], Line, Tlen, _, _) ->
    yystate(66, Ics, Line, Tlen+1, 0, Tlen);
yystate(6, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(57, Ics, Line, Tlen+1, 0, Tlen);
yystate(6, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(57, Ics, Line, Tlen+1, 0, Tlen);
yystate(6, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(57, Ics, Line, Tlen+1, 0, Tlen);
yystate(6, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(57, Ics, Line, Tlen+1, 0, Tlen);
yystate(6, Ics, Line, Tlen, _, _) ->
    {0,Tlen,Ics,Line,6};
yystate(5, [123|Ics], Line, Tlen, _, _) ->
    yystate(69, Ics, Line, Tlen+1, 16, Tlen);
yystate(5, [59|Ics], Line, Tlen, _, _) ->
    yystate(39, Ics, Line, Tlen+1, 16, Tlen);
yystate(5, [42|Ics], Line, Tlen, _, _) ->
    yystate(24, Ics, Line, Tlen+1, 16, Tlen);
yystate(5, [41|Ics], Line, Tlen, _, _) ->
    yystate(108, Ics, Line, Tlen+1, 16, Tlen);
yystate(5, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(5, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 40 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(5, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 58 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(5, [C|Ics], Line, Tlen, _, _) when C >= 60, C =< 122 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(5, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(44, Ics, Line, Tlen+1, 16, Tlen);
yystate(5, Ics, Line, Tlen, _, _) ->
    {16,Tlen,Ics,Line,5};
yystate(4, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(4, [117|Ics], Line, Tlen, _, _) ->
    yystate(68, Ics, Line, Tlen+1, 21, Tlen);
yystate(4, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(4, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(4, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(4, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 116 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(4, [C|Ics], Line, Tlen, _, _) when C >= 118, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(4, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(4, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,4};
yystate(3, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(3, [105|Ics], Line, Tlen, _, _) ->
    yystate(67, Ics, Line, Tlen+1, 21, Tlen);
yystate(3, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(3, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(3, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(3, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 104 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(3, [C|Ics], Line, Tlen, _, _) when C >= 106, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(3, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(3, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,3};
yystate(2, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [102|Ics], Line, Tlen, _, _) ->
    yystate(114, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [32|Ics], Line, Tlen, _, _) ->
    yystate(2, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [9|Ics], Line, Tlen, _, _) ->
    yystate(2, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 8 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 31 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [C|Ics], Line, Tlen, _, _) when C >= 33, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 101 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [C|Ics], Line, Tlen, _, _) when C >= 103, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(2, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,2};
yystate(1, [123|Ics], Line, Tlen, _, _) ->
    yystate(19, Ics, Line, Tlen+1, 21, Tlen);
yystate(1, [99|Ics], Line, Tlen, _, _) ->
    yystate(58, Ics, Line, Tlen+1, 21, Tlen);
yystate(1, [42|Ics], Line, Tlen, _, _) ->
    yystate(115, Ics, Line, Tlen+1, 21, Tlen);
yystate(1, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(1, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(1, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 98 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(1, [C|Ics], Line, Tlen, _, _) when C >= 100, C =< 122 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(1, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(86, Ics, Line, Tlen+1, 21, Tlen);
yystate(1, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,1};
yystate(0, [123|Ics], Line, Tlen, _, _) ->
    yystate(45, Ics, Line, Tlen+1, 21, Tlen);
yystate(0, [42|Ics], Line, Tlen, _, _) ->
    yystate(63, Ics, Line, Tlen+1, 21, Tlen);
yystate(0, [C|Ics], Line, Tlen, _, _) when C >= 0, C =< 9 ->
    yystate(45, Ics, Line, Tlen+1, 21, Tlen);
yystate(0, [C|Ics], Line, Tlen, _, _) when C >= 11, C =< 41 ->
    yystate(45, Ics, Line, Tlen+1, 21, Tlen);
yystate(0, [C|Ics], Line, Tlen, _, _) when C >= 43, C =< 122 ->
    yystate(45, Ics, Line, Tlen+1, 21, Tlen);
yystate(0, [C|Ics], Line, Tlen, _, _) when C >= 124 ->
    yystate(45, Ics, Line, Tlen+1, 21, Tlen);
yystate(0, Ics, Line, Tlen, _, _) ->
    {21,Tlen,Ics,Line,0};
yystate(S, Ics, Line, Tlen, Action, Alen) ->
    {Action,Alen,Tlen,Ics,Line,S}.

%% yyaction(Action, TokenLength, TokenChars, TokenLine) ->
%% {token,Token} | {end_token, Token} | skip_token | {error,String}.
%% Generated action function.

yyaction(0, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_0(TokenChars, TokenLine);
yyaction(1, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_1(TokenChars, TokenLine);
yyaction(2, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_2(TokenChars, TokenLine);
yyaction(3, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_3(TokenChars, TokenLine);
yyaction(4, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_4(TokenChars, TokenLine);
yyaction(5, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_5(TokenChars, TokenLine);
yyaction(6, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_6(TokenChars, TokenLine);
yyaction(7, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_7(TokenChars, TokenLine);
yyaction(8, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_8(TokenChars, TokenLine);
yyaction(9, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_9(TokenChars, TokenLine);
yyaction(10, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_10(TokenChars, TokenLine);
yyaction(11, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_11(TokenChars, TokenLine);
yyaction(12, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_12(TokenChars, TokenLine);
yyaction(13, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_13(TokenChars, TokenLine);
yyaction(14, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_14(TokenChars, TokenLine);
yyaction(15, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_15(TokenChars, TokenLine);
yyaction(16, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_16(TokenChars, TokenLine);
yyaction(17, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_17(TokenChars, TokenLine);
yyaction(18, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_18(TokenChars, TokenLine);
yyaction(19, _, _, _) ->
    yyaction_19();
yyaction(20, _, _, _) ->
    yyaction_20();
yyaction(21, TokenLen, YYtcs, TokenLine) ->
    TokenChars = yypre(YYtcs, TokenLen),
    yyaction_21(TokenChars, TokenLine);
yyaction(22, _, _, _) ->
    yyaction_22();
yyaction(_, _, _, _) -> error.

-compile({inline,yyaction_0/2}).
-file("src/smart_move_leex.xrl", 93).
yyaction_0(TokenChars, TokenLine) ->
     { token, { comment, TokenLine, TokenChars } } .

-compile({inline,yyaction_1/2}).
-file("src/smart_move_leex.xrl", 94).
yyaction_1(TokenChars, TokenLine) ->
     { token, { comment_start, TokenLine, TokenChars } } .

-compile({inline,yyaction_2/2}).
-file("src/smart_move_leex.xrl", 95).
yyaction_2(TokenChars, TokenLine) ->
     { token, { comment_end, TokenLine, TokenChars } } .

-compile({inline,yyaction_3/2}).
-file("src/smart_move_leex.xrl", 96).
yyaction_3(TokenChars, TokenLine) ->
     { token, { comment, TokenLine, TokenChars } } .

-compile({inline,yyaction_4/2}).
-file("src/smart_move_leex.xrl", 99).
yyaction_4(TokenChars, TokenLine) ->
     { token, { module, TokenLine, TokenChars } } .

-compile({inline,yyaction_5/2}).
-file("src/smart_move_leex.xrl", 101).
yyaction_5(TokenChars, TokenLine) ->
     { token, { script, TokenLine, TokenChars } } .

-compile({inline,yyaction_6/2}).
-file("src/smart_move_leex.xrl", 103).
yyaction_6(TokenChars, TokenLine) ->
     { token, { use, TokenLine, trim (TokenChars) } } .

-compile({inline,yyaction_7/2}).
-file("src/smart_move_leex.xrl", 104).
yyaction_7(TokenChars, TokenLine) ->
     { token, { const, TokenLine, trim (TokenChars) } } .

-compile({inline,yyaction_8/2}).
-file("src/smart_move_leex.xrl", 105).
yyaction_8(TokenChars, TokenLine) ->
     { token, { friend, TokenLine, trim (TokenChars) } } .

-compile({inline,yyaction_9/2}).
-file("src/smart_move_leex.xrl", 107).
yyaction_9(TokenChars, TokenLine) ->
     { token, { struct_begin, TokenLine, trim (TokenChars) } } .

-compile({inline,yyaction_10/2}).
-file("src/smart_move_leex.xrl", 108).
yyaction_10(TokenChars, TokenLine) ->
     { token, { struct, TokenLine, trim (TokenChars) } } .

-compile({inline,yyaction_11/2}).
-file("src/smart_move_leex.xrl", 111).
yyaction_11(TokenChars, TokenLine) ->
     { token, { function, TokenLine, trim (TokenChars) } } .

-compile({inline,yyaction_12/2}).
-file("src/smart_move_leex.xrl", 112).
yyaction_12(TokenChars, TokenLine) ->
     { token, { fun_begin, TokenLine, trim (TokenChars) } } .

-compile({inline,yyaction_13/2}).
-file("src/smart_move_leex.xrl", 114).
yyaction_13(TokenChars, TokenLine) ->
     { token, { other, TokenLine, TokenChars } } .

-compile({inline,yyaction_14/2}).
-file("src/smart_move_leex.xrl", 115).
yyaction_14(TokenChars, TokenLine) ->
     { token, { lbrace_line, TokenLine, TokenChars } } .

-compile({inline,yyaction_15/2}).
-file("src/smart_move_leex.xrl", 116).
yyaction_15(TokenChars, TokenLine) ->
     { token, { lbrace, TokenLine, TokenChars } } .

-compile({inline,yyaction_16/2}).
-file("src/smart_move_leex.xrl", 117).
yyaction_16(TokenChars, TokenLine) ->
     { token, { rbrace, TokenLine, parse_brace (TokenChars) } } .

-compile({inline,yyaction_17/2}).
-file("src/smart_move_leex.xrl", 118).
yyaction_17(TokenChars, TokenLine) ->
     { token, { rbrace_semicolon, TokenLine, parse_brace (TokenChars) } } .

-compile({inline,yyaction_18/2}).
-file("src/smart_move_leex.xrl", 119).
yyaction_18(TokenChars, TokenLine) ->
     { token, { annotation, TokenLine, trim (TokenChars) } } .

-compile({inline,yyaction_19/0}).
-file("src/smart_move_leex.xrl", 120).
yyaction_19() ->
     skip_token .

-compile({inline,yyaction_20/0}).
-file("src/smart_move_leex.xrl", 121).
yyaction_20() ->
     skip_token .

-compile({inline,yyaction_21/2}).
-file("src/smart_move_leex.xrl", 122).
yyaction_21(TokenChars, TokenLine) ->
     { token, { other, TokenLine, TokenChars } } .

-compile({inline,yyaction_22/0}).
-file("src/smart_move_leex.xrl", 123).
yyaction_22() ->
     skip_token .

-file("/Users/cam/.asdf/installs/erlang/24.2.1/lib/parsetools-2.3.2/include/leexinc.hrl", 313).
